@@ -15,13 +15,12 @@
 #
 # END HEADER
 
-"""This module provides the core primitives of Hypothesis, assume and given."""
+"""This module provides the core primitives of Hypothesis, such as given."""
 
 
 from __future__ import division, print_function, absolute_import
 
 import time
-import inspect
 import functools
 import traceback
 from random import Random
@@ -37,7 +36,7 @@ from hypothesis.executors import new_style_executor, \
     default_new_style_executor
 from hypothesis.reporting import report, verbose_report, current_verbosity
 from hypothesis.statistics import note_engine_for_statistics
-from hypothesis.internal.compat import getargspec, str_to_bytes
+from hypothesis.internal.compat import str_to_bytes, getfullargspec
 from hypothesis.internal.escalation import \
     escalate_hypothesis_internal_error
 from hypothesis.internal.reflection import nicerepr, arg_string, \
@@ -72,8 +71,7 @@ Example = namedtuple('Example', ('args', 'kwargs'))
 
 
 def example(*args, **kwargs):
-    """Add an explicit example called with these args and kwargs to the
-    test."""
+    """A decorator to that ensures a specific example is always tested."""
     if args and kwargs:
         raise InvalidArgument(
             'Cannot mix positional and keyword arguments for examples'
@@ -155,10 +153,8 @@ def is_invalid_test(
         return invalid(
             'given must be called with at least one argument')
 
-    if (
-        generator_arguments and (
-            original_argspec.varargs or original_argspec.keywords)
-    ):
+    if (generator_arguments and (original_argspec.varargs or
+                                 original_argspec.varkw)):
         return invalid(
             'varargs or keywords are not supported with positional '
             'arguments to @given'
@@ -178,8 +174,10 @@ def is_invalid_test(
             'cannot mix positional and keyword arguments to @given'
         )
     extra_kwargs = [
-        k for k in generator_kwargs if k not in original_argspec.args]
-    if extra_kwargs and not original_argspec.keywords:
+        k for k in generator_kwargs
+        if k not in original_argspec.args + original_argspec.kwonlyargs
+    ]
+    if extra_kwargs and not original_argspec.varkw:
         return invalid(
             '%s() got an unexpected keyword argument %r' % (
                 name,
@@ -192,32 +190,36 @@ def is_invalid_test(
                 'destructuring arguments') % (
                     name,
             ))
-    if original_argspec.defaults:
-        return invalid(
-            'Cannot apply @given to a function with defaults.'
-        )
+    if original_argspec.defaults or original_argspec.kwonlydefaults:
+        return invalid('Cannot apply @given to a function with defaults.')
+    missing = [repr(kw) for kw in original_argspec.kwonlyargs
+               if kw not in generator_kwargs]
+    if missing:
+        raise InvalidArgument('Missing required kwarg{}: {}'.format(
+            's' if len(missing) > 1 else '', ', '.join(missing)))
 
 
 def execute_explicit_examples(
     test_runner, test, wrapped_test, settings, arguments, kwargs
 ):
-    original_argspec = getargspec(test)
+    original_argspec = getfullargspec(test)
 
     for example in reversed(getattr(
         wrapped_test, 'hypothesis_explicit_examples', ()
     )):
+        example_kwargs = dict(original_argspec.kwonlydefaults or {})
         if example.args:
             if len(example.args) > len(original_argspec.args):
                 raise InvalidArgument(
                     'example has too many arguments for test. '
                     'Expected at most %d but got %d' % (
                         len(original_argspec.args), len(example.args)))
-            example_kwargs = dict(zip(
+            example_kwargs.update(dict(zip(
                 original_argspec.args[-len(example.args):],
                 example.args
-            ))
+            )))
         else:
-            example_kwargs = example.kwargs
+            example_kwargs.update(example.kwargs)
         if Phase.explicit not in settings.phases:
             continue
         example_kwargs.update(kwargs)
@@ -242,6 +244,9 @@ def execute_explicit_examples(
 
 
 def fail_health_check(settings, message, label):
+    # Tell pytest to omit the body of this function from tracebacks
+    # http://doc.pytest.org/en/latest/example/simple.html#writing-well-integrated-assertion-helpers
+    __tracebackhide__ = True
     if label in settings.suppress_health_check:
         return
     message += (
@@ -256,6 +261,8 @@ def fail_health_check(settings, message, label):
 
 
 def perform_health_checks(random, settings, test_runner, search_strategy):
+    # Tell pytest to omit the body of this function from tracebacks
+    __tracebackhide__ = True
     if not settings.perform_health_check:
         return
     if not Settings.default.perform_health_check:
@@ -468,6 +475,8 @@ class StateForActualGivenExecution(object):
             data.mark_interesting()
 
     def run(self):
+        # Tell pytest to omit the body of this function from tracebacks
+        __tracebackhide__ = True
         database_key = str_to_bytes(fully_qualified_name(self.test))
         start_time = time.time()
         runner = ConjectureRunner(
@@ -567,15 +576,14 @@ def given(*given_arguments, **given_kwargs):
     """A decorator for turning a test function that accepts arguments into a
     randomized test.
 
-    This is the main entry point to Hypothesis. See the full tutorial
-    for details of its behaviour.
+    This is the main entry point to Hypothesis.
 
     """
     def run_test_with_generator(test):
         generator_arguments = tuple(given_arguments)
         generator_kwargs = dict(given_kwargs)
 
-        original_argspec = getargspec(test)
+        original_argspec = getfullargspec(test)
 
         check_invalid = is_invalid_test(
             test.__name__, original_argspec,
@@ -586,23 +594,27 @@ def given(*given_arguments, **given_kwargs):
 
         arguments = original_argspec.args
 
-        for name, strategy in zip(
-            arguments[-len(generator_arguments):], generator_arguments
-        ):
+        for name, strategy in zip(arguments[-len(generator_arguments):],
+                                  generator_arguments):
             generator_kwargs[name] = strategy
 
-        argspec = inspect.ArgSpec(
+        argspec = original_argspec._replace(
             args=[a for a in arguments if a not in generator_kwargs],
-            keywords=original_argspec.keywords,
-            varargs=original_argspec.varargs,
-            defaults=None
+            defaults=None,
         )
+        annots = {k: v for k, v in argspec.annotations.items()
+                  if k in (argspec.args + argspec.kwonlyargs)}
+        annots['return'] = None
+        argspec = argspec._replace(annotations=annots)
 
         @impersonate(test)
         @define_function_signature(
             test.__name__, test.__doc__, argspec
         )
         def wrapped_test(*arguments, **kwargs):
+            # Tell pytest to omit the body of this function from tracebacks
+            __tracebackhide__ = True
+
             settings = wrapped_test._hypothesis_internal_use_settings
 
             random = get_random_for_wrapped_test(test, wrapped_test)
@@ -648,6 +660,8 @@ def given(*given_arguments, **given_kwargs):
 
 
 def find(specifier, condition, settings=None, random=None, database_key=None):
+    """Returns the minimal example from the given strategy ``specifier`` that
+    matches the predicate function ``condition``."""
     settings = settings or Settings(
         max_examples=2000,
         min_satisfying_examples=0,
