@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2015 David R. MacIver
+# Most of this work is copyright (C) 2013-2017 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -24,7 +24,7 @@ import tempfile
 import unicodedata
 
 from hypothesis.configuration import tmpdir, storage_directory
-from hypothesis.internal.compat import hunichr, GzipFile, FileExistsError
+from hypothesis.internal.compat import GzipFile, FileExistsError, hunichr
 
 
 def charmap_file():
@@ -33,6 +33,7 @@ def charmap_file():
         'charmap.pickle.gz'
     )
 
+
 _charmap = None
 
 
@@ -49,7 +50,7 @@ def charmap():
                     rs[-1][-1] += 1
                 else:
                     rs.append([i, i])
-            # We explicitly set the mtime to an arbitary value so as to get
+            # We explicitly set the mtime to an arbitrary value so as to get
             # a stable format for our charmap.
             data = sorted(
                 (k, tuple((map(tuple, v))))
